/*********************************************************************

 Author        : ADI - Apps            www.analog.com/MicroConverter

 Date          : Dec. 2006

 File          : WakeUpTimer.c

 Hardware      : Applicable to ADuC703x

 Description   : Wake-up Timer causes part to power-up every 8s.
 				 the timer is configured in Hr:Min:Sec:Hundredths format
				
				Additional (Gilbert): Can be woke up by LIN during power down mode.

				 T2LD:
				 bits 31-24 represent the hours
				 bits 21-16 represent the minutes
				 bits 13-8 represent the secondes
				 bits 6-0 represent the hundredths

				 The debugger cannot be used when the part is placed in power down mode.
		
*********************************************************************/

#include <ADuC7032.h>

void LIN_INIT(void);
void My_FIQ_Function(void);
void My_IRQ_Function(void);


int main()
{

	//LIN_INIT();

	FIQ = My_FIQ_Function; 	// Initialise Interrupt Routine	 for LIN
	IRQ = My_IRQ_Function;  // Initialise Interrupt Routine	 for Timer2

	IRQEN = 0x010;			// Enable T2 (Wake-up Timer) to generate an interrupt
//	FIQEN = 0x80;			// Generate an IRQ on LIN break

//	GP2DAT = 0x02000000;
//	GP0DAT = 0x01000000;				//Enable P0.0 (GPIO_0) as an output

	T2LD = 0x800;						// Wait x seconds in power down mode
	T2VAL = 0x800;
	T2CON = 0x2E8;						// Clock Source - Low Power Oscilator
										// Count down, Enable Timer 2, hours mode
	while(1){}							// Infinite Loop
}



void My_IRQ_Function() 
{
	int iIRQ_Source;
	unsigned long it1=1234, it2=4321;
	iIRQ_Source = IRQSTA;				// Save a temporary copy of IRQSTA
  	if ((iIRQ_Source &
	 0x010)!=0)
	{
		//GP2DAT ^=0x020000;					// Complement P2.1
//		GP0DAT ^=0x010000;

		T2CLRI = 0x01;					// restart Timer 2
		POWKEY0 = 0x01;		 			// Put all peripherals, PLL and Core Asleep
		POWCON  = 0x00;					// default value is 79	- 20MHz
		POWKEY1 = 0xF4;					// 
		it1 = it1*it2;					// Dummy cycle to ensure correct power down
		it1 = it1*it2;
		//T2CLRI = 0x01;					// restart Timer 2
	}	 	
	return ;
}


void LIN_INIT(void )
{
char HVstatus;

	// Enable LHS on GPIO Pins 
	GP2CON = 0x110000;

	LHSCON0 = 0x1;							// Reset LHS Interface

	do										// Enable LIN
	{
		HVDAT = 0x02; 						// Enable Normal LIN TX mode 
		HVCON = 0x08; 						// Write to Config0
		do
		{
			HVstatus = HVCON;
		}
		while(HVstatus & 0x1); 				// Wait until command is finished
	}
	while (!(HVstatus & 0x4));				// transmit command is correct

	while((LHSSTA & 0x20) == 0 )
	{										// Wait until the LHS Hardware is reset
	}

	LHSCON1 = 0x062;						// Sets Stop Edge as the fifth falling edge
											// and the start edge as the first falling
											// edge in the sync byte
	LHSCON0 = 0x0114;						// Gates UART RX Line, ensure no interference 
											// from the LIN into the UART.
											// Selects the stop condition as a falling edge
											// Enables Generation of an interrupt on the 
											// stop condition.
											// Enables the interface
	LHSVAL1 = 0x030;						// Set number of 131kHz periods to generate a Break Interrupt
											// 0x3F / 131kHz ~ 480us which is just over 9.5 TBits.
											// According to LIN V2.0, a slave with a clock accuracy of <2% may
											// use a break detection of 9.5 bit, otherwise a break detection of
											// 11 bits is required
											// This is for 19200bps
}



void My_FIQ_Function() 
{
	int iFIQ_Source,itemp,iDL,iDL_Power,iDL_temp;
//	unsigned char Data_To_Tranmist[8];

	iFIQ_Source = FIQSTA;					// Save a temporary copy of IRQSTA		

	if ( ( iFIQ_Source  & 0x80) !=0)		// LIN Interrupt
	{
		itemp = LHSSTA;		
		if ((itemp & 0x1 ) != 0 )			// LIN Break Symbol
		{
 
		}
		if ((itemp & 0x2 ) != 0 )			// LIN Start Condition
		{

		}
		if ((itemp & 0x4 ) != 0 )			// LIN Stop Condition
		{
			// Configure UART
 	
			// 	DL	=	LHSVAL0 >>	CD_Bits + 6
			//	For explanation please refer to technote

			iDL = LHSVAL0 >> (( POWCON & 0x7)+6);

			iDL_Power = 0;
			iDL_temp = iDL;
			while(iDL >> (iDL_Power +1 ))
			{
				iDL_Power++;
			}
			// If using the fractional divider:
			iDL_temp = iDL_Power + (POWCON & 0x7);
			if ( iDL_temp > 5 )
			{
				iDL_temp= (LHSVAL0 >> ( iDL_temp -5 )) - 2048; 
			}
			else
			{
				iDL_temp= (LHSVAL0 << (5 - iDL_temp )) - 2048; 
			}
			COMDIV2 = 0x8800 + iDL_temp;

			COMCON0 = 0x080;			// Set DLAB
			// Setting DIV0 and DIV1 to DL calculated
			COMDIV0 = (1<< iDL_Power ) & 0xff;
   			COMDIV1 = (1<< iDL_Power ) & 0xff00;
  		 	COMCON0 = 0x03;				// Clear DLAB
			COMIEN0 = 0x1;			    // Enable RX interrupt only

			while((GP2DAT & 0x10 ) == 0 )
			{
			}
			LHSCON0 = 0x4;				// Enable LHS to detect Break Condition
										// Ungate RX Line
										// Disable all interrupts, less Break
			FIQEN = 0x800;				// Enable UART Interrupts

		// The UART is now configured and ready to be used for LIN
		
		}
	}
	if ( ( iFIQ_Source  & 0x800) !=0)	// UART Interrupt
	{
		// Remember, each time you transmit a byte, you will also receive
		// it as the LIN bus connects TX and RX
		FIQCLR = 0x800;				// Disable UART Interrupts
		itemp = ( COMRX & 0x3f );	// Do not care about Parity bits


	

		LIN_INIT();					// Re initialise LHS

 	 }
	 	
	 return;
}
